package http

import (
	"io/ioutil"
	"net/http"
	"testing"
	"time"

	"github.com/stretchr/testify/assert"
	fullenv "xbus.io/xbus/fullenv_tests"
)

func TestFullenv(t *testing.T) {
	env, _ := fullenv.NewFullEnvForTest(t, "fullenv", "fullenv_test.yaml")
	defer env.Close()
	if !assert.NoError(t, env.Init()) {
		t.FailNow()
	}
	if err := env.Startup(); err != nil {
		t.Fatal(err)
	}
	defer func() { assert.NoError(t, env.Shutdown()) }()
	time.Sleep(600 * time.Millisecond)
	resp, err := http.Get("http://localhost:8911/test/processrequest")
	if err != nil {
		t.Fatal(err)
	}
	if resp.StatusCode != 404 {
		t.Fatal("Expected a 404 error")
	}
	resp, err = http.Get("http://localhost:8911/test/processrequest?key=test")
	if err != nil {
		t.Fatal(err)
	}
	if resp.StatusCode != 404 {
		t.Fatal("Expected a 404 error")
	}
	data, err := ioutil.ReadAll(resp.Body)
	assert.NoError(t, err)
	assert.Equal(t, `{"error": "nats: authorization violation"}`, string(data))
}
