// Copyright 2017 Orus.io. All rights reserved.

package sql

import (
	"testing"

	"github.com/slicebit/qb"

	"xbus.io/go-xbus/v3/api"
	"xbus.io/xbus/storage/sql/model"
	"xbus.io/xbus/testutils"
	"xbus.io/xbus/testutils/sqldb"
)

func TestLog(t *testing.T) {
	logger := testutils.GetTestLogger(t)
	defer logger.VerifyExpectations()

	testDB := sqldb.GetTestDB(t)
	if testDB == nil {
		return
	}
	defer testDB.Close()

	m, db := testDB.GetModelDB(1)
	defer db.Close()

	logStorage := NewLogStorage(m, db, *logger.Zero())
	db.Engine.SetLogFlags(qb.LQuery | qb.LBindings)

	t.Run("Basics", func(t *testing.T) {
		defer logger.SubTestCapture(t)()
		if _, err := db.Engine.DB().Exec("DELETE FROM log_entry"); err != nil {
			t.Fatal(err)
		}
		logentry := api.LogEntry{
			Message: api.LogMessage{Text: "test"},
		}

		logStorage.Log(logentry)

	})

	t.Run("DeleteActor#125", func(t *testing.T) {
		defer logger.SubTestCapture(t)()
		if _, err := db.Engine.DB().Exec("DELETE FROM actor"); err != nil {
			t.Fatal(err)
		}
		if _, err := db.Engine.DB().Exec("DELETE FROM log_entry"); err != nil {
			t.Fatal(err)
		}

		actor := model.Actor{
			Name: "Test",
			Kind: api.Actor_EMITTER,
		}
		if err := db.Insert(&actor); err != nil {
			t.Fatal(err)
		}

		logentry := api.LogEntry{
			ActorID: api.UUID(actor.ID),
			Message: api.LogMessage{Text: "test"},
		}

		logStorage.Log(logentry)

		actorStorage := NewActorStorage(m, db, *logger.Zero())

		actorStorage.Delete(logentry.ActorID)
	})

	t.Run("DeleteEnvelope", func(t *testing.T) {
		defer logger.SubTestCapture(t)()
		if _, err := db.Engine.DB().Exec("DELETE FROM log_entry"); err != nil {
			t.Fatal(err)
		}
		if _, err := db.Engine.DB().Exec("DELETE FROM envelope"); err != nil {
			t.Fatal(err)
		}

		envelope := model.Envelope{}
		if err := db.Insert(&envelope); err != nil {
			t.Fatal(err)
		}

		logentry := api.LogEntry{
			EnvelopeID: api.UUID(envelope.ID),
			Message:    api.LogMessage{Text: "test"},
		}

		logStorage.Log(logentry)

		envStorage := NewEnvelopeStorage(m, db, *logger.Zero())

		envStorage.Purge(func(ids []api.UUID) []api.UUID { return ids })
	})
}
